<?php

namespace FiloBlu\Paymentscapture\Block\Adminhtml\System\Config;

/**
 * Renderer for Scheduled operations
 */

class Scheduledevents extends \Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray
{

    const ADYEN_HPP_METHOD_CODE = 'adyen_hpp';

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\Status\Collection
     */
    protected $collection;

    /**
     * Statuses per state array
     *
     * @var array
     */
    protected $stateStatuses;

    /**
     * @var array
     */
    private $statuses;

    /**
     * @var Status
     */
    protected $orderStatusFactory;

    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory
     */
    protected $orderStatusCollectionFactory;

    /**
     * @var \Magento\Payment\Api\PaymentMethodListInterface
     */
    protected $methodList;

    /**
     * @var Payment Methods
     */
    protected $paymentMethodRenderer = null;

    /**
     * @var Initial status
     */
    protected $initialStatusRenderer = null;

    /**
     * @var \FiloBlu\Paymentscapture\Helper\Data
     */
    protected $paymentscaptureHelper;

    /**
     * Scheduledevents constructor.
     * @param \Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory $orderStatusCollectionFactory
     * @param \Magento\Payment\Api\PaymentMethodListInterface $methodList
     * @param \FiloBlu\Paymentscapture\Helper\Data $paymentscaptureHelper
     * @param \Magento\Backend\Block\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Magento\Sales\Model\ResourceModel\Order\Status\CollectionFactory $orderStatusCollectionFactory,
        \Magento\Payment\Api\PaymentMethodListInterface $methodList,
        \FiloBlu\Paymentscapture\Helper\Data $paymentscaptureHelper,
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ) {
        $this->orderStatusCollectionFactory = $orderStatusCollectionFactory;
        $this->methodList=$methodList;
        $this->paymentscaptureHelper = $paymentscaptureHelper;
        parent::__construct($context, $data);
    }

    /**
     * Enable the "Add after" button or not
     *
     * @var bool
     */
    protected $_addAfter = true;

    /**
     * Rows cache
     *
     * @var array|null
     */
    private $_arrayRowsCache;

    /**
     * Prepare to render
     *
     * @return void
     */
    protected function _prepareToRender()
    {

        $statuses = $this->orderStatusCollectionFactory->create()->toOptionHash();
        $statuses = array('*' => __('-- Any --')) + $statuses;

        $processingStatuses = $this->orderStatusCollectionFactory->create()->addStateFilter('processing')->toOptionHash();
        $processingStatuses = array('*' => __('-- Any --')) + $processingStatuses;

        $initialStatusBlock = $this->getInitialStatusRenderer($statuses);
        $this->addColumn('initial_status', array(
            'label' => __('Initial Status'),
            'renderer' => $initialStatusBlock
        ));

        $paymentMethodBlock = $this->getPaymentMethodRenderer();
        $this->addColumn('payment_methods', array(
            'label' => __('Payment Methods'),
            'renderer' => $paymentMethodBlock
        ));

        $this->addColumn('delay', array(
            'label' => __('Delay (in minutes)')
        ));

        $captureRequiredBlock = $this->getLayout()->createBlock('Magento\Framework\View\Element\Html\Select')
            ->setId($this->_getCellInputElementId('<%- _id %>','is_capture_required'))
            ->setName($this->_getCellInputElementName('is_capture_required'))
            ->setClass('select')
            ->setOptions(
                array(
                    ['value' => '0', 'label' => __('No')],
                    ['value' => '1', 'label' => __('Yes')],
                )
            );

        $this->addColumn('is_capture_required',array(
            'label' => __('Capture Order'),
            'renderer' => $captureRequiredBlock,
        ));

        $finalstatusesBlock=$this->getLayout()->createBlock('Magento\Framework\View\Element\Html\Select')
            ->setId($this->_getCellInputElementId('<%- _id %>','final_status'))
            ->setName($this->_getCellInputElementName('final_status'))
            ->setClass('select')
            ->setOptions($processingStatuses);

        $this->addColumn('final_status', array(
            'label' => __('Final Status'),
            'renderer' => $finalstatusesBlock
        ));

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add operation');
    }

    protected function _prepareArrayRow(\Magento\Framework\DataObject $row)
    {
        $renderer=  $this->getPaymentMethodRenderer();

        //The phtml set the value of DOM element by doing a $().value
        //This does not work with array values. Therefore we remove the values from column_value array.
        //The values is set by "option_extra_attrs" set in the following.
        $column_values=$row->getColumnValues();
        unset($column_values[$row->getData("_id")."_payment_methods"]);
        $row->setData("column_values", $column_values);

        //Set selected values for multiselect using option_extra_attrs
        $payment_methods = $row->getPaymentMethods();
        if($payment_methods){
            $options = [];
            foreach ($payment_methods as $payment_method) {
                $options['option_' . $renderer->calcOptionHash($payment_method)]
                    = 'selected="selected"';
            }
            $row->setData('option_extra_attrs', $options);
        }

        //error_log(print_r($row,true),3, "/app/elisabetta-franchi-2.0/var/log/debug_event.log");
    }

    protected function getPaymentMethodRenderer()
    {
        if (!$this->paymentMethodRenderer) {
            $paymentMethodObjects=$this->methodList->getList(0);
            $paymentMethods=array(array('value' => '*', 'label' =>  __('-- Any --')));
            foreach($paymentMethodObjects as $paymentObject){
                if ($paymentObject->getCode() == self::ADYEN_HPP_METHOD_CODE) { //going to add HPP methods later
                    continue;
                }
                array_push($paymentMethods, array("value" => $paymentObject->getCode(), "label" => $paymentObject->getCode()));
            }

            //load also HPP methods
            $paymentMethods = array_merge($paymentMethods, $this->paymentscaptureHelper->getHppMethodList());
            //error_log(print_r($this->paymentscaptureHelper->getHppMethodList(),true),3, "/app/elisabetta-franchi-2.0/var/log/debug_event.log");

            $this->paymentMethodRenderer = $this->getLayout()->createBlock(
                'Magento\Framework\View\Element\Html\Select',
                '',
                ['data' => ['is_render_to_js_template' => true]]
            )
                ->setId($this->_getCellInputElementId('<%- _id %>','payment_methods'))
                ->setName($this->_getCellInputElementName('payment_methods')."[]")
                ->setClass('multiselect')
                ->setOptions($paymentMethods)
                ->setExtraParams('style="width:100px" multiple="multiple" size="5" ');
        }

        return $this->paymentMethodRenderer;
    }

    protected function getInitialStatusRenderer($statuses) {
        if (!$this->initialStatusRenderer) {
            $orderStatuses = array();
            foreach($statuses as $status_value => $status_label){
                array_push($orderStatuses, array("value" => $status_value, "label" => $status_label));
            }
            $this->initialStatusRenderer = $this->getLayout()->createBlock(
                'Magento\Framework\View\Element\Html\Select',
                '',
                ['data' => ['is_render_to_js_template' => true]]
            )
                ->setId($this->_getCellInputElementId('<%- _id %>','initial_status'))
                ->setName($this->_getCellInputElementName('initial_status')."[]")
                ->setClass('multiselect')
                ->setOptions($orderStatuses)
                ->setExtraParams('style="width:100px" multiple="multiple" size="5" ');
        }
        return $this->initialStatusRenderer;
    }
 
}
