<?php

namespace FiloBlu\Paymentscapture\Observer;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\Event\ObserverInterface;


 class checkAndCapture implements ObserverInterface{

     const EVENT_BASED_OPERATIONS_CONFIG_PATH = 'filoblu_paymentscapture/general/event_based_operations';
     const CAPTURE_ACTIVE_CONFIG_PATH = 'filoblu_paymentscapture/general/active';
     const CAPTURE_DEBUG_CONFIG_PATH = 'filoblu_paymentscapture/general/debug';
     const DEBUG_LOGFILE = 'paymentscapture_capture.log';
    
	/**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;

	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \FiloBlu\Paymentscapture\Helper\Data
     */  
    protected $_helper;

     /**
      * @var \Magento\Store\Model\StoreManagerInterface
      */
    protected $_storeManager;

     /**
      * @var \FiloBlu\Paymentscapture\Model\Capture
      */
    protected $_capturer;


     /**
      * checkAndCapture constructor.
      * @param \Psr\Log\LoggerInterface $logger
      * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
      * @param \FiloBlu\Paymentscapture\Helper\Data $helper
      * @param \Magento\Store\Model\StoreManagerInterface $storeManager
      * @param \FiloBlu\Paymentscapture\Model\Capture $capturer
      */
	public function __construct(
            \Psr\Log\LoggerInterface $logger,
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \FiloBlu\Paymentscapture\Helper\Data $helper,
            \Magento\Store\Model\StoreManagerInterface $storeManager,
            \FiloBlu\Paymentscapture\Model\Capture $capturer
	){
            $this->_logger = $logger;
            $this->_scopeConfig = $scopeConfig;
            $this->_helper = $helper;
            $this->_storeManager = $storeManager;
            $this->_capturer = $capturer;
	}


    public function execute(Observer $observer){

        $order = $observer->getEvent()->getOrder();

        if (!$this->_scopeConfig->isSetFlag(self::CAPTURE_ACTIVE_CONFIG_PATH, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $order->getStoreId())) {
            return;
        }

        $store = $this->_storeManager->getStore($order->getStoreId());
        $websiteId = $store->getWebsiteId();
        $eventOperations = $this->_helper->getSerializedConfigValue(self::EVENT_BASED_OPERATIONS_CONFIG_PATH, $websiteId);
        if (is_null($eventOperations) || empty($eventOperations)) {
            return;
        }

        foreach ($eventOperations as $eventOperation) {

            //check payment method
            $orderPaymentMethod = array('code' => $order->getPayment()->getMethod(), 'cctype' => $order->getPayment()->getCcType());
            if (!$this->_helper->paymentMethodMatches($orderPaymentMethod, $eventOperation['payment_methods'])) {
                continue;
            }
            //check if capture is needed ? capture
            $isCaptureRequired = $eventOperation["is_capture_required"];
            if ($isCaptureRequired) {
                $isCaptured = $this->_capturer->_captureOnlinePayment($order);
            } else {
                $isCaptured = TRUE;
            }
            //check if order status (after capture) is ok with final status
            if ($isCaptured) {
                $finalStatus = ($eventOperation['final_status'] && $eventOperation['final_status'] != '*') ? $eventOperation['final_status'] : FALSE;
                if ($finalStatus != FALSE && $order->getStatus() != $finalStatus) {
                    $order->setState('processing');
                    $order->addStatusHistoryComment("Order processed by Payment capture event observer", $finalStatus);
                    $order->save();
                } else {
                    $order->addStatusHistoryComment("Order processed by Payment capture event observer (no status modification needed)");
                    $order->save();
                }
            } else {
                $this->debugLog('Order #' . $order->getIncrementId() . ' not captured');
            }
        }

        return TRUE;
    }

     protected function debugLog($message) {
         if ($this->_scopeConfig->isSetFlag(self::CAPTURE_DEBUG_CONFIG_PATH)) {
             $writer = new \Zend\Log\Writer\Stream(BP . '/var/log/' . self::DEBUG_LOGFILE);
             $logger = new \Zend\Log\Logger();
             $logger->addWriter($writer);
             $logger->info($message);
         }
     }
}